'use strict';

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const AutoMuteToggleButton = GObject.registerClass(
    {GTypeName: 'AutoMuteToggleButton'},
    class AutoMuteToggleButton extends PanelMenu.Button {
        _init() {
            super._init(0.0, Me.metadata.name, false);

            this.cards = [];
            this.autoMute = true;
            this.loadCards();

            this.icon = new St.Icon({
                gicon: new Gio.ThemedIcon({name: 'audio-headphones-symbolic'}),
                style_class: 'system-status-icon',
            });

            this.add_child(this.icon);
            this.connect('button-press-event', this.onPress.bind(this));
        }

        async loadCards() {
            this.cards.length = 0;
            const cards = await this.listCards();

            for (let i = 0; i < cards.length; i++) {
                if (await this.hasAutoMuteToggle(cards[i])) {
                    this.cards.push(cards[i]);
                }
            }
        }

        async listCards() {
            const output = await this.fetchStdout(['/usr/bin/aplay', '-l']);

            const cards = output.match(/card \d+:/g);
            const unique = cards.filter((v, k, a) => !k || v !== a[k-1]);
            const numbers = unique.map(v => v.match(/\d+/)[0]);

            return numbers;
        }

        async hasAutoMuteToggle(card) {
            const output = await this.fetchStdout(['/usr/bin/amixer', '-c', card, 'controls']);

            return output.match(/Auto-Mute Mode/);
        }

        fetchStdout(argv) {
            const sub = new Gio.Subprocess({
                argv,
                flags: Gio.SubprocessFlags.STDOUT_PIPE,
            });

            sub.init(null);

            return new Promise(resolve => {
                sub.communicate_utf8_async(null, null, (obj, result) => {
                    resolve(obj.communicate_utf8_finish(result)[1]);
                });
            });
        }

        onPress() {
            this.autoMute = !this.autoMute;
            this.updateMixer();
            this.updateIcon();
        }

        updateMixer() {
            const value = this.autoMute ? 'Enabled' : 'Disabled';

            this.cards.forEach(card => {
                GLib.spawn_async(
                    '/',
                    ['/usr/bin/amixer', '-c', card, 'set', 'Auto-Mute Mode', value],
                    ['LANG=C'],
                    GLib.SpawnFlags.STDOUT_TO_DEV_NULL,
                    null,
                );
            })
        }

        updateIcon() {
            this.icon.icon_name = this.autoMute
                ? 'audio-headphones-symbolic'
                : 'audio-card-symbolic';
        }
    },
);

let button = null;

function enable() {
    button = new AutoMuteToggleButton();
    Main.panel.addToStatusArea(Me.metadata.name, button, 'right');
}

function disable() {
    if (button !== null) {
        button.destroy();
        button = null;
    }
}

